#!/usr/bin/env python
# coding: utf-8

# # 简介
# 
# 赛题链接： 

# ## 赛事背景
# 
# 对于移动设备厂商而言，获取当前手机用户的人口属性信息是非常困难的。基于用户的手机及日常使用应用程序的偏好准确地预测其人口属性信息是提升个性化体验、构建精准用户画像的基础。需要说明的是，本赛事数据已获得个人用户的充分认可和同意，并已进行适当的匿名处理以保护隐私。由于保密，我们不会提供有关如何获得性别和年龄数据的详细信息。
# 
# 
# ## 赛事任务
# 
# 本次比赛有两个任务，分别对移动设备（device_id）进行性别和年龄的预测，这里包含二分类和回归两个问题，最终会将两个部分的分数结合起来进行排名。
# 
# ## 评审规则
# 
# 1. 数据说明
# 
# 赛题数据由训练集、测试集、事件数据组成。总设备id超过2w，包含设备信息、APP信息和事件信息，其中device_id为用户的唯一标识符，gender为用户性别，age为用户年龄。为了保证比赛的公平性，将会从中抽取2w条设备id为训练集，3千多个设备id作为测试集，同时会对部分字段信息进行脱敏。
# 
# 
# <img src="./pic/data.png" width = "300" height = "150" alt="data" align=center />
#  
# 
# 2. 评估指标
# 
# 本次竞赛的评价标准由两部分组成，性别预测使用准确率指标，年龄预测使用1/(MAE+1)，最高分为2。评估代码参考：

# In[ ]:


from sklearn.metrics import accuracy_score,mean_absolute_error
# standard 为标准答案
# submit 为选手提交答案
score1 = accuracy_score(standard['gender'], submit['gender'])
score2 = 1 / (1 + mean_absolute_error(standard['age'], submit['age']))
# 最终分数
final_score = score1 + score2


# 
# 3. 评测及排行
# 
# 1、赛事提供下载数据，选手在本地进行算法调试，在比赛页面提交结果。
# 
# 2、每支团队每天最多提交3次。
# 
# 3、排行按照得分从高到低排序，排行榜将选择团队的历史最优成绩进行排名。
# 
# ## 作品提交要求
# 
# 文件格式：预测结果文件按照csv格式
# 
# 提交文件大小：无要求
# 
# 提交次数限制：每支队伍每周最多3次
# 
# 预测结果文件详细说明：
# 
# 1) 以csv格式提交，编码为UTF-8，第一行为表头；
# 
# 2) 提交前请确保预测结果的格式与sample_submit.csv中的格式一致。具体格式如下：

# In[ ]:


device_id,gender,age

20000,0,0

20001,0,0

20002,0,0

20003,0,0

20004,0,0


# ## 赛程规则
# 
# 正式赛
# 
# 8月16日——9月15日
# 
# 初赛截止成绩以团队在初赛时间段内最优成绩为准（不含测试排名）。
# 
# 初赛作品提交截止日期为9月8日17:00；正式赛名次公布日期为9月9日10:00。
# 
# 长期赛
# 
# 9月16日——10月24日
# 
# 因赛事以学习实践为主，正式赛将转变为长期赛，供开发者学习实践。本阶段提交后，系统会根据成绩持续更新榜单，但该阶段榜单不再进行公示和奖励。
# 
# 六、奖项设置
# 
# 本赛题设立一、二、三等奖各一名，具体详情如下：
# 
# 一等奖：1支队伍，周赛一等奖证书，奖金：1000元
# 
# 二等奖：1支队伍，周赛二等奖证书，奖金：800元
# 
# 三等奖：1支队伍，周赛三等奖证书，奖金：500元

# # 赛题Baseline
# 
# ## 导入常用工具包&读取数据
# 
# ### 常用工具包导入
# 
# 

# In[ ]:


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import os 
import gc
import glob
from joblib import Parallel, delayed
import seaborn as sns
from tqdm import tqdm
from sklearn.model_selection import KFold 
import lightgbm as lgbm  


# 
# ### 数据读取

# In[ ]:


data_path = './data/' 
df_tr               = pd.read_csv(data_path + 'train.csv')
df_tr_app_events    = pd.read_csv(data_path + 'train_app_events.csv')
df_te               = pd.read_csv(data_path + 'test.csv')
df_te_app_events    = pd.read_csv(data_path + 'test_app_events.csv') 


# In[ ]:


df_tr_te_app_events = pd.concat([df_tr_app_events,df_te_app_events],axis=0,ignore_index = True) 


# ## 数据分析

# In[ ]:


df_tr_te_app_events.head()


# ### gender
# 
# - gender一共有两个不同的值，0和1，分布看起来相对均匀

# In[ ]:


df_tr['gender'].value_counts().plot(kind = 'bar')


# 
# <img src="./pic/gender_distribution.png" width = "400" height = "150" alt="gender_distribution" align=center />
# 

# ### age
# 
# - age主要分布在20-40岁之间

# In[ ]:


plt.figure(figsize = [10,8])
sns.distplot(df_tr['age'])


# 
# <img src="./pic/age_distribution.png" width = "400" height = "150" alt="age_distribution" align=center />
# 

# ### date
# 
# - date的分布较为均匀

# In[ ]:


df_tr_te_app_events['date'].value_counts().plot(kind = 'bar')


# ### event_id
# 
# - event_id的分布并不均衡，有很多值为1

# In[ ]:


df_tr_te_app_events['event_id'].value_counts()


# ### app_id
# 
# - app_id的较小的出现次数较多。

# In[ ]:


sns.distplot(df_tr_te_app_events['app_id'])


# In[ ]:


df_tr_te_app_events['app_id'].value_counts()


# ### 小结
# 
# 从上面的基础分析来看，本次赛题的数据基本符合我们的直观理解。至于其它的细节，大家可以根据自己的需求进行分析。下面我们构建该赛题的Baseline。

# ## 模型构建
# ### 特征工程
# 
# #### tag_list的长度

# In[ ]:


df_tr_te_app_events['tag_list_len'] = df_tr_te_app_events['tag_list'].apply(lambda x: x.count(',')+1)


# #### 基础特征工程
# 
# - event_id：出现次数&不同值；
# - app_id：出现的不同值；
# - is_installed：均值和和；
# - is_active：均值和和；
# - date：最大值，不同值；
# - tag_list_len：均值和标准差

# In[ ]:


agg_dic = {
    "event_id":['count','nunique'],
    "app_id":['nunique'], 
    "is_installed":[np.mean,np.sum],
    "is_active":[np.mean,np.sum],
    "date":[np.max, 'nunique'],
    "tag_list_len":[np.mean, np.std] 
}

df_device_features = df_tr_te_app_events.groupby('device_id').agg(agg_dic).reset_index()

fea_names = ['_'.join(c) for c in df_device_features.columns]
df_device_features.columns = fea_names
df_device_features.rename(columns = {'device_id_':'device_id'},inplace = True)


# In[ ]:


df_device_features.head()


# #### 特征组合
# 
# - 常用的count/nunqiue & mean/std组合

# In[ ]:


df_device_features['event_id_count_nunique_ratio'] = df_device_features['event_id_count'] / df_device_features['event_id_nunique']
df_device_features['tag_list_len_mean_div_std'] = df_device_features['tag_list_len_mean'] / df_device_features['tag_list_len_std']


# #### 特征拼接

# In[ ]:


df_tr = df_tr.merge(df_device_features, on = 'device_id' , how='left')
df_te = df_te.merge(df_device_features, on = 'device_id' , how='left')


# In[ ]:


del df_tr_te_app_events
gc.collect()


# In[ ]:


df_tr.head()


# In[ ]:


df_tr.columns


# ## 模型训练&预测
# 
# ### 特征&标签设计

# In[ ]:


tr_features = ['device_id',  'phone_brand', 'device_model', 'event_id_count', 'event_id_nunique', 'app_id_nunique',
       'is_installed_mean', 'is_installed_sum', 'is_active_mean', 'is_active_sum', 'date_amax', 'date_nunique', 'tag_list_len_mean',
       'tag_list_len_std', 'event_id_count_nunique_ratio','tag_list_len_mean_div_std']
label_gender   = 'gender'
label_age      = 'age'


# ### gender模型

# In[ ]:


y_valid.value_counts()


# In[ ]:


lgb_params = {
      "objective": "binary", 
      "metric": "binary_error", 
      "boosting_type": "gbdt",
      'early_stopping_rounds': 100,
      'learning_rate': 0.01,  
      'colsample_bytree':0.95, 
} 

X_tr_val = df_tr[tr_features + [label_gender]]
X_te     = df_te[tr_features]
 
kf = KFold(n_splits=5)
lgb_gender_models = []
y_pred = 0
for f,(tr_ind,val_ind) in enumerate(kf.split(X_tr_val)):
    
    X_train,X_valid = X_tr_val.iloc[tr_ind][tr_features], X_tr_val.iloc[val_ind][tr_features]
    y_train,y_valid = X_tr_val.iloc[tr_ind][label_gender], X_tr_val.iloc[val_ind][label_gender]
    
    lgbm_train = lgbm.Dataset(X_train,y_train)  
    lgbm_valid = lgbm.Dataset(X_valid,y_valid)

    model_binary = lgbm.train(params=lgb_params, 
                  train_set=lgbm_train,
                  valid_sets=[lgbm_train, lgbm_valid],
                  num_boost_round=100000,   
                  verbose_eval=100)
    y_pred += model_binary.predict(X_te[tr_features]) / 5.0
    lgb_gender_models.append(model_binary) 


# In[ ]:


df_submit = df_te[['device_id']].copy()
df_submit['gender'] = (y_pred >=0.5) + 0
df_submit['gender'] = df_submit['gender'].astype(int)


# ### age模型

# In[ ]:


from sklearn.metrics import mean_absolute_error
def feval_lgb_Age(preds, lgbm_train):
    
    labels =lgbm_train.get_label()
    return 'Age Error', round(1.0 / (1.0 + mean_absolute_error(y_true = labels, y_pred = preds)),7), True


lgb_params = {
      "objective": "mae",  
      "boosting_type": "gbdt",
      'early_stopping_rounds': 100,
      'learning_rate': 0.01,  
      'colsample_bytree':0.95, 
} 

X_tr_val = df_tr[tr_features + [label_age]]
X_te     = df_te[tr_features]
 
kf = KFold(n_splits=5)
lgb_age_models = []
y_pred = 0
for f,(tr_ind,val_ind) in enumerate(kf.split(X_tr_val)):
    
    X_train,X_valid = X_tr_val.iloc[tr_ind][tr_features], X_tr_val.iloc[val_ind][tr_features]
    y_train,y_valid = X_tr_val.iloc[tr_ind][label_age], X_tr_val.iloc[val_ind][label_age]
    
    lgbm_train = lgbm.Dataset(X_train,y_train)  
    lgbm_valid = lgbm.Dataset(X_valid,y_valid)

    model_mae = lgbm.train(params=lgb_params, 
                  train_set=lgbm_train,
                  valid_sets=[lgbm_train, lgbm_valid],
                  num_boost_round=100000,   
                     feval = feval_lgb_Age,
                  verbose_eval=100)
    y_pred += model_mae.predict(X_te[tr_features]) / 5.0
    lgb_age_models.append(model_mae) 


# In[ ]:


df_submit['age'] = y_pred


# ### 模型提交

# In[ ]:


df_submit.to_csv('baseline_Fold5_lgb.csv',index = None)


# # 结论
# 
# 本文我们对数据进行了简单的分析并且设计了简单的Baseline，目前模型都还是一些基础的统计特征，可以提升的地方非常多，例如：
# 
# - 加强特征工程：比如tag list数据如何充分的利用好；设计序列化的信息来强化用户的刻画；
# - 该问题是多目标优化的问题，可以参考Wechat和腾讯之前的比赛，使用一些最新的技术，例如MMOE等进行联合优化；
# - 设计NN和树模型等融合



#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import pandas as pd
from autogluon.timeseries import TimeSeriesDataFrame, TimeSeriesPredictor


# In[ ]:


df = pd.read_csv("https://autogluon.s3.amazonaws.com/datasets/timeseries/m4_hourly_subset/train.csv")
df.head()


# In[ ]:


train_data = TimeSeriesDataFrame.from_data_frame(
    df,
    id_column="item_id",
    timestamp_column="timestamp"
)
train_data.head()


# In[ ]:


predictor = TimeSeriesPredictor(
    prediction_length=48,
    path="autogluon-m4-hourly",
    target="target",
    eval_metric="MASE",
)

predictor.fit(
    train_data,
    presets="medium_quality",
    time_limit=600,
)


# In[ ]:


predictions = predictor.predict(train_data)
predictions.head()


# In[ ]:


import matplotlib.pyplot as plt

# TimeSeriesDataFrame can also be loaded directly from a file
test_data = TimeSeriesDataFrame.from_path("https://autogluon.s3.amazonaws.com/datasets/timeseries/m4_hourly_subset/test.csv")

# Plot 4 randomly chosen time series and the respective forecasts
predictor.plot(test_data, predictions, quantile_levels=[0.1, 0.9], max_history_length=200, max_num_item_ids=4);


# In[ ]:


# The test score is computed using the last
# prediction_length=48 timesteps of each time series in test_data
predictor.leaderboard(test_data)




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import json
import pandas as pd 
import numpy as np
from prophet import Prophet
from pandas.plotting import register_matplotlib_converters
import warnings
import datetime
from scipy import stats
#import statsmodels.api as sm
import matplotlib.pyplot as plt
from sklearn import preprocessing
from sklearn.metrics import mean_squared_error,mean_absolute_error,r2_score
get_ipython().run_line_magic('matplotlib', 'inline')
#from fbprophet.plot import add_changepoints_to_plot
#from fbprophet.diagnostics import cross_validation
#from fbprophet.plot import plot_cross_validation_metric
#from fbprophet.diagnostics import performance_metrics
from prophet.diagnostics import cross_validation
from prophet.plot import add_changepoints_to_plot
from prophet.diagnostics import cross_validation
from prophet.plot import plot_cross_validation_metric
from prophet.diagnostics import performance_metrics
warnings.filterwarnings("ignore")
import os
#os.environ['NUMEXPR_MAX_THREADS'] = '64'

from neuralprophet import NeuralProphet, set_log_level


# In[ ]:


###best 0.7+0.3

new_train_data_best= pd.read_csv('best_1.9.csv')
new_train_data_prophet = pd.read_csv('predict_day17_1_best_3.35973.csv')
new_data_2in1_1  = new_train_data_best*0.7 + new_train_data_prophet*0.3

new_data_2in1_1.to_csv('D20_加权_best_prophet_Trynew_1.csv',index=False) #2.85



# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:


###best 0.7+0.3
'''
new_train_data_best= pd.read_csv('best_1.9.csv')
new_train_data_prophet = pd.read_csv('predict_day17_1_best_3.35973.csv')
new_data_2in1_1  = new_train_data_best*0.7 + new_train_data_prophet*0.3

new_data_2in1_1.to_csv('D20_加权_best_prophet_Trynew_1.csv',index=False)
'''




import setuptools
from setuptools import setup, find_packages

with open("README.md","r",encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='yaoguaimi',  # 替换为你项目的名称
    version='0.1.2',
    include_package_data=True,
    author='hzx',
    author_email='hzx802502@gmail.com',
    description='A package with Jupyter Notebook code for machine learning tasks',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),  # 自动发现项目中的包
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    install_requires=[
    ],
)



import torch
import torch.nn as nn
from functools import partial
import numpy as np
from functools import partial
import math

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
class DropPath(nn.Module):
    """Drop paths (Stochastic Depth) per sample."""
    def __init__(self, drop_prob=None):
        super().__init__()
        self.drop_prob = drop_prob

    def forward(self, x):
        return drop_path(x, self.drop_prob, self.training)

def drop_path(x, drop_prob: float = 0., training: bool = False):
    """Drop paths (Stochastic Depth) per sample (when applied in main path of residual blocks)."""
    if drop_prob == 0. or not training:
        return x
    keep_prob = 1 - drop_prob
    # work with diff dim tensors, not just 2D ConvNets
    shape = (x.shape[0],) + (1,) * (x.ndim - 1)
    random_tensor = keep_prob + torch.rand(shape, dtype=x.dtype, device=x.device)
    random_tensor.floor_()  # binarize
    output = x.div(keep_prob) * random_tensor
    return output

class MLPMixerLayer(nn.Module):
    def __init__(self, num_patches, hidden_dim, tokens_mlp_dim, channels_mlp_dim,dropout_rate=0,drop_path_rate=0.25):
        super().__init__()
        self.norm1 = nn.LayerNorm(hidden_dim)
        self.token_mixing = nn.Sequential(
            nn.Linear(num_patches, tokens_mlp_dim),
            nn.GELU(),
            nn.Dropout(dropout_rate),  # Add dropout here
            nn.Linear(tokens_mlp_dim, num_patches),
        )

        self.norm2 = nn.LayerNorm(hidden_dim)
        self.channel_mixing = nn.Sequential(
            nn.Linear(hidden_dim, channels_mlp_dim),
            nn.GELU(),
            nn.Dropout(dropout_rate),  # Add dropout here
            nn.Linear(channels_mlp_dim, hidden_dim),
        )

        self.drop_path = DropPath(drop_path_rate) if drop_path_rate > 0 else nn.Identity()

    def forward(self, x):
        # Apply LayerNorm before token mixing
        y = self.norm1(x)
        # Token mixing (patch mixing)
        y = y.permute(0, 2, 1)  # Swap the 'hidden_dim' and 'num_patches' dimensions
        y = self.token_mixing(y)
        y = y.permute(0, 2, 1)  # Swap back after token mixing
        x = x + self.drop_path(y)  # DropPath 应用于残差连接
        # Apply LayerNorm before channel mixing      
        y = self.norm2(x)
        # Channel mixing
        y = self.channel_mixing(y)
        x = x + self.drop_path(y)  # 再次应用
        return x


        
class Neural_receiver(nn.Module):
    def __init__(self, num_patches, num_layers, hidden_dim, tokens_mlp_dim, channels_mlp_dim, num_symbols, subcarriers, streams, num_bits_per_symbol,dropout_rate=0):
        super().__init__()
        self.streams = streams
        self.num_bits_per_symbol = num_bits_per_symbol
        self.num_symbols = num_symbols
        self.subcarriers = subcarriers
        self.num_patches = num_symbols * subcarriers
        self.patch_embedding = nn.Sequential(
            nn.Linear(streams * 6, hidden_dim),
            nn.Dropout(0)  # Add dropout here
        )
        self.mixer_layers = nn.ModuleList([
            MLPMixerLayer(num_patches, hidden_dim, tokens_mlp_dim, channels_mlp_dim) for _ in range(num_layers)
        ])
        self.layer_norm = nn.LayerNorm(hidden_dim)
        self.head = nn.Sequential(
            nn.Dropout(0),  # Add dropout here before the final linear layer
            nn.Linear(hidden_dim, streams * num_bits_per_symbol)
        )

    def forward(self, y, template_pilot):
        batch_size = y.shape[0]
        samples = y.shape[0]
        ###########################STD标准化#############################
        #y_mean = y.mean(dim=(0, 2, 3), keepdim=True)  # 对样本、符号和子载波的维度求均值
        #y_std = y.std(dim=(0, 2, 3), keepdim=True)    # 对样本、符号和子载波的维度求标准差
        #y_normalized = (y - y_mean) / (y_std + 1e-6)  # 防止除以零
        # 标准化导频 P
        #P_mean = template_pilot.mean(dim=(0, 2, 3), keepdim=True)  # 对符号和子载波的维度求均值
        #P_std = template_pilot.std(dim=(0, 2, 3), keepdim=True)    # 对符号和子载波的维度求标准差
        #P_normalized = (template_pilot - P_mean) / (P_std + 1e-6)  # 防止除以零
        ###########################除以最大值标准化#############################
       # y_max_abs = y.abs().max(dim=0, keepdim=True)[0].max(dim=1, keepdim=True)[0].max(dim=1, keepdim=True)[0]  # Fix applied here
       # y_normalized = y / (y_max_abs + 1e-6)  # 避免除以零
       # p_max_abs = template_pilot.abs().max(dim=0, keepdim=True)[0].max(dim=1, keepdim=True)[0].max(dim=1, keepdim=True)[0]  # Fix applied here
      #  p_normalized = template_pilot / (p_max_abs + 1e-6)  # 避免除以零
        ###############################功率归一化#############################
        # 假定 y 是你的信号张量，形状为 [20000, 4, 12, 96, 2]
        #y_complex = torch.view_as_complex(y)  # 将最后一个维度转换为复数
        #signal_power = torch.mean(torch.abs(y_complex)**2, dim=[1, 2, 3], keepdim=True)  # 计算每个样本的功率        
        #target_power = 1.0  # 假定你希望每个样本的平均功率是 1
        #normalization_factor = torch.sqrt(signal_power / target_power)  # 计算归一化因子
        #y_normalized = y_complex / normalization_factor  # 应用归一化
        #y_normalized = torch.view_as_real(y_normalized)  # 如果需要，转换回实部和虚部表示

        # 假定 P_expanded 是扩展后的导频信号，形状为 [20000, 4, 12, 96, 2]
        #P_expanded_complex = torch.view_as_complex(template_pilot)  # 转换为复数形式
        # 计算每个样本的导频功率
       #P_expanded_power = torch.mean(torch.abs(P_expanded_complex)**2, dim=(1, 2, 3), keepdim=True)  # 对所有维度求平均功率，除了批次维度        
        # 计算归一化因子
        #normalization_factor = torch.sqrt(P_expanded_power / target_power)       
        # 应用功率归一化
        #P_expanded_normalized_complex = P_expanded_complex / normalization_factor  # 应用归一化       
        # 转换回实部和虚部表示
       # P_expanded_normalized = torch.view_as_real(P_expanded_normalized_complex)
        # 先计算每个样本的均值，并进行中心化
        # 假定 rx_signal 是你的接收信号张量，形状为 [20000, 4, 12, 96, 2]
        # 假定 pilot 是你的导频信号张量，形状为 [20000, 4, 12, 96, 2]
        # 将最后一个维度（实部和虚部）转换为复数

 ##########################################功率归一化2###########################################      
        # 设置目标功率
        #target_power = 1.0  # 例如，我们将目标功率设置为1        
        # 假定 rx_signal 和 pilot 是你的接收信号和导频信号张量，形状为 [20000, 4, 12, 96, 2]        
        # 将最后一个维度（实部和虚部）转换为复数
        #rx_signal_complex = torch.view_as_complex(y)
        #pilot_complex = torch.view_as_complex(template_pilot)      
        # 对接收信号进行中心化
        #rx_signal_mean = torch.mean(rx_signal_complex, dim=[1, 2, 3], keepdim=True)
        #rx_signal_centered = rx_signal_complex - rx_signal_mean       
        # 对导频信号进行中心化
        #pilot_mean = torch.mean(pilot_complex, dim=[1, 2, 3], keepdim=True)
        #pilot_centered = pilot_complex - pilot_mean        
        # 计算接收信号的归一化因子，并应用归一化
        #rx_signal_power = torch.mean(torch.abs(rx_signal_centered)**2, dim=[1, 2, 3], keepdim=True)
        #normalization_factor_rx = torch.sqrt(rx_signal_power / target_power)  # 注意这里的除法，我们将信号功率除以目标功率
        #rx_signal_normalized = rx_signal_centered / (normalization_factor_rx + 1e-6)  # 防止除以零
        #rx_signal_normalized = torch.view_as_real(rx_signal_normalized)  # 转换回实部和虚部表示      
        # 计算导频信号的归一化因子，并应用归一化
        #pilot_power = torch.mean(torch.abs(pilot_centered)**2, dim=[1, 2, 3], keepdim=True)
        #normalization_factor_pilot = torch.sqrt(pilot_power / target_power)  # 同样将信号功率除以目标功率
        #pilot_normalized = pilot_centered / (normalization_factor_pilot + 1e-6)  # 防止除以零
        #pilot_normalized = torch.view_as_real(pilot_normalized)  # 转换回实部和虚部表示
        # 对y进行归一化处理

 ##########################################功率归一化3###########################################      
        
        # 假定 rx_signal 和 pilot 是你的接收信号和导频信号张量，形状为 [batch_size, channels, height, width, 2]
        
        # 将最后一个维度（实部和虚部）转换为复数
        rx_signal_complex = torch.view_as_complex(y)
        pilot_complex = torch.view_as_complex(template_pilot)
        
        # 计算整个批次的平均功率
        rx_signal_power = torch.mean(torch.abs(rx_signal_complex) ** 2)
        pilot_power = torch.mean(torch.abs(pilot_complex) ** 2)
        
        # 设置目标功率
        target_power = 1.0  # 例如，我们将目标功率设置为1
        
        # 计算归一化因子
        normalization_factor_rx = torch.sqrt(rx_signal_power / target_power)
        normalization_factor_pilot = torch.sqrt(pilot_power / target_power)
        
        # 应用功率归一化
        rx_signal_normalized_complex = rx_signal_complex / (normalization_factor_rx + 1e-6)  # 防止除以零
        pilot_normalized_complex = pilot_complex / (normalization_factor_pilot + 1e-6)  # 防止除以零
        
        # 转换回实部和虚部表示
        rx_signal_normalized = torch.view_as_real(rx_signal_normalized_complex)
        pilot_normalized = torch.view_as_real(pilot_normalized_complex)
                     
        
       # y_max_abs = y.abs().max()  # 获取y中所有元素的最大绝对值
        #y_normalized = y / (y_max_abs + 1e-6)  # 对整个批次的数据进行归一化
        
        # 对template_pilot进行归一化处理
        #p_max_abs = template_pilot.abs().max()  # 获取template_pilot中所有元素的最大绝对值
        #p_normalized = template_pilot / (p_max_abs + 1e-6)  # 对整个批次的数据进行归一化
        def MMSE_channel_estimation(rx_signal, pilot, pilot_positions, noise_variance, power_ratio):
            """
            :param rx_signal: 接收信号张量
            :param pilot: 导频张量
            :param pilot_positions: 导频位置数组
            :param noise_variance: 噪声方差
            :param power_ratio: 导频与数据功率比
            :return: 估计的信道张量
            """
            # 获取张量的形状
            num_samples, num_antennas, num_symbols, num_subcarriers, _ = rx_signal.size()
        
            # 初始化估计的信道张量
            estimated_channel = torch.zeros(num_samples, num_antennas, num_symbols, num_subcarriers, 2)
        
            # 计算每个导频的功率比
            pilot_power = power_ratio / (power_ratio + 1)
            data_power = 1 / (power_ratio + 1)
        
            # 对每一层每个导频位置进行信道估计
            for layer in range(len(pilot_positions)):  # 确保循环通过所有层
                for symbol in pilot_positions[layer]:
                    # 应用MMSE估计
                    for subcarrier in range(num_subcarriers):
                        # 对每个样本和每个接收天线
                        H_hat = (rx_signal[:, :, symbol, subcarrier, :] * pilot_power) / \
                                (pilot_power * pilot_power + noise_variance)
                        estimated_channel[:, :, symbol, subcarrier, :] = H_hat
            
            return estimated_channel
        
        # 定义导频位置和功率比
        pilot_positions = [[0, 4, 8], [1, 5, 9], [2, 6, 10], [3, 7, 11]]
        power_ratio = 1.6 / 0.6  # 导频与数据的功率比
        noise_variance = 0.1  # 假设噪声方差，需要根据实际情况调整
        y = y.to(device)
        template_pilot = template_pilot.to(device)

        # 执行信道估计
        estimated_channel = MMSE_channel_estimation(y, template_pilot, pilot_positions, noise_variance, power_ratio)
        
        # 现在可以使用estimated_channel进行后续处理
        estimated_channel = estimated_channel.to(device)

        
        y = torch.cat([y,template_pilot,estimated_channel], dim=-1)  # Concatenating pilot signals to the input #template_pilot     
        y = y.view(batch_size, self.num_patches, -1)  # Reshaping into patches
        y = self.patch_embedding(y)  # Apply patch embedding
        # Apply mixer layers
        for layer in self.mixer_layers:
            y = layer(y)
        # Apply final normalization and classification head
        y = self.layer_norm(y)
        y = self.head(y)
        y = y.view(batch_size,  self.streams, self.num_symbols,self.subcarriers ,self.num_bits_per_symbol)
        return y



#!/usr/bin/env python
# coding: utf-8

# <font color=black size=5 face=雅黑>**Step 1:导入函数工具箱**</font>
# 

# In[ ]:


from sklearn.decomposition import PCA #主成分分析法
import pandas as pd 
import numpy as np
from prophet import Prophet
from pandas.plotting import register_matplotlib_converters
from sklearn import preprocessing
import matplotlib.pyplot as plt
import seaborn as sns
from preprocessing import filterData, delData, fillNanList, dropDuplicate,\
    handleOutlier, minMaxScale, cate2Num,standardizeData,\
    discrete, tran_math_function, minMaxScale, standardizeData,\
    onehot_map, map_dict_tran, binary_map, pca_selection,dfs_feature,\
    continue_time, discrete_time, statistics_time
from sklearn.metrics import mean_squared_error,mean_absolute_error,r2_score
from statsmodels.tsa.stattools import *
from statsmodels.graphics.tsaplots import plot_acf, plot_pacf
from sklearn.linear_model import Ridge
from sklearn import linear_model


# <font color=black size=5 face=雅黑>**Step 2:读取数据**</font>
# 

# In[ ]:


#new_train_data = pd.read_csv('datasets/@huangzx23#3eb493098a9258fca92f4758cbbc0dcd/E_test.csv')
new_train_data = pd.read_csv('/home/bwfy/桌面/比赛赛集/E_test.csv')
A_train = pd.read_csv('/home/bwfy/桌面/比赛赛集/A.csv')
B_train = pd.read_csv('/home/bwfy/桌面/比赛赛集/B.csv')
C_train = pd.read_csv('/home/bwfy/桌面/比赛赛集/C.csv')
D_train = pd.read_csv('/home/bwfy/桌面/比赛赛集/D.csv')
new_train_data.head()


# <font color=black size=5 face=雅黑>**Step 3:数据预处理**</font>
# 

# <font color=black size=3 face=雅黑>**3.1:ADF平稳性检测**</font>
# 

# In[ ]:


new_train_data_1 =new_train_data.copy()


# In[ ]:


result1 = adfuller(new_train_data_1['AvgTemperature'])
result2 = adfuller(A_train['AvgTemperature'])
result3 = adfuller(B_train['AvgTemperature'])
result4 = adfuller(C_train['AvgTemperature'])
result5 = adfuller(D_train['AvgTemperature'])
print(result1)
print(result2)
print(result3)
print(result4)
print(result5)


# In[ ]:


print(result1)


# <font color=black size=3 face=雅黑>**Step 3.2:自相关图平稳性swim**</font>

# In[ ]:


fig = plt.figure(figsize=(20, 10))
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(223)
ax3 = fig.add_subplot(224)

ax1.plot(new_train_data_1['AvgTemperature'])
ax1.set_title('E_test')
plot_acf(new_train_data_1['AvgTemperature'], ax=ax2)
plot_pacf(new_train_data_1['AvgTemperature'], ax=ax3)
plt.show()


# In[ ]:


fig = plt.figure(figsize=(20, 10))
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(223)
ax3 = fig.add_subplot(224)

ax1.plot(A_train['AvgTemperature'])
ax1.set_title('A_train')
plot_acf(A_train['AvgTemperature'], ax=ax2)
plot_pacf(A_train['AvgTemperature'], ax=ax3)
plt.show()


# In[ ]:


fig = plt.figure(figsize=(20, 10))
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(223)
ax3 = fig.add_subplot(224)

ax1.plot(B_train['AvgTemperature'])
ax1.set_title('B_train')
plot_acf(B_train['AvgTemperature'], ax=ax2)
plot_pacf(B_train['AvgTemperature'], ax=ax3)
plt.show()


# In[ ]:


fig = plt.figure(figsize=(20, 10))
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(223)
ax3 = fig.add_subplot(224)

ax1.plot(C_train['AvgTemperature'])
ax1.set_title('C_train')
plot_acf(C_train['AvgTemperature'], ax=ax2)
plot_pacf(C_train['AvgTemperature'], ax=ax3)
plt.show()


# In[ ]:


fig = plt.figure(figsize=(20, 10))
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(223)
ax3 = fig.add_subplot(224)

ax1.plot(D_train['AvgTemperature'])
ax1.set_title('D_train')
plot_acf(D_train['AvgTemperature'], ax=ax2)
plot_pacf(D_train['AvgTemperature'], ax=ax3)
plt.show()


# <font color=black size=3 face=雅黑>**3.2:Box-Pierce检验平稳性swim**</font>

# In[ ]:


from statsmodels.stats.diagnostic import acorr_ljungbox
# 随机种子
np.random.seed(123)
# 生成白噪声序列
#white_noise=np.random.standard_normal(size=1000)
# 白噪声检验，想要查看QBP检验统计量的结果需要指定boxpierce参数为True
# 不指定boxpierce参数，默认返回QLB检验统计量的结果
res = acorr_ljungbox(new_train_data_1['AvgTemperature'], lags=24, boxpierce=True, return_df=True)
print(res)

# 根据检验统计量值查表
# from scipy import stats
# print(stats.chi2.sf(18.891607, 20))
#我们这里只看检验统计量下检验效果，即最后两列。可以可看到，各延迟阶数下的p值均小于0.05，拒绝原假设，序列非白噪声。


# <font color=black size=3 face=雅黑>**3.3 异常值处理**</font>

# In[ ]:


#格式化各数据集
new_train_data_1['ds']=new_train_data_1['Year'].astype('str').str.cat(new_train_data_1['Month'].astype('str'),sep='-')
new_train_data_1['ds']=new_train_data_1['ds'].astype('str').str.cat(new_train_data_1['Day'].astype('str'),sep='-')

A_train['ds']=A_train['Year'].astype('str').str.cat(A_train['Month'].astype('str'),sep='-')
A_train['ds']=A_train['ds'].astype('str').str.cat(A_train['Day'].astype('str'),sep='-')

B_train['ds']=B_train['Year'].astype('str').str.cat(B_train['Month'].astype('str'),sep='-')
B_train['ds']=B_train['ds'].astype('str').str.cat(B_train['Day'].astype('str'),sep='-')

C_train['ds']=C_train['Year'].astype('str').str.cat(C_train['Month'].astype('str'),sep='-')
C_train['ds']=C_train['ds'].astype('str').str.cat(C_train['Day'].astype('str'),sep='-')

D_train['ds']=D_train['Year'].astype('str').str.cat(D_train['Month'].astype('str'),sep='-')
D_train['ds']=D_train['ds'].astype('str').str.cat(D_train['Day'].astype('str'),sep='-')

import datetime
new_train_data_1['ds']=new_train_data_1['ds'].map(lambda x:datetime.datetime.strptime(x, '%Y-%m-%d').strftime('%Y-%m-%d '))
A_train['ds']=A_train['ds'].map(lambda x:datetime.datetime.strptime(x, '%Y-%m-%d').strftime('%Y-%m-%d '))
B_train['ds']=B_train['ds'].map(lambda x:datetime.datetime.strptime(x, '%Y-%m-%d').strftime('%Y-%m-%d '))
C_train['ds']=C_train['ds'].map(lambda x:datetime.datetime.strptime(x, '%Y-%m-%d').strftime('%Y-%m-%d '))
D_train['ds']=D_train['ds'].map(lambda x:datetime.datetime.strptime(x, '%Y-%m-%d').strftime('%Y-%m-%d '))

new_train_data_1.drop(["Month","Day","Year"],axis=1,inplace=True)
A_train.drop(["Month","Day","Year"],axis=1,inplace=True)
B_train.drop(["Month","Day","Year"],axis=1,inplace=True)
C_train.drop(["Month","Day","Year"],axis=1,inplace=True)
D_train.drop(["Month","Day","Year"],axis=1,inplace=True)


new_train_data_1.rename(columns={'AvgTemperature':'y'}, inplace=True)#将原来的命名修改为算法要求的命名
A_train.rename(columns={'AvgTemperature':'y'}, inplace=True)#将原来的命名修改为算法要求的命名
B_train.rename(columns={'AvgTemperature':'y'}, inplace=True)#将原来的命名修改为算法要求的命名
C_train.rename(columns={'AvgTemperature':'y'}, inplace=True)#将原来的命名修改为算法要求的命名
D_train.rename(columns={'AvgTemperature':'y'}, inplace=True)#将原来的命名修改为算法要求的命名


new_train_data_1.head()


# In[ ]:


print(new_train_data_1)


# In[ ]:


new_train_data_1 = new_train_data_1.drop_duplicates(subset=['ds']) #丢弃重复
A_train = A_train.drop_duplicates(subset=['ds']) #丢弃重复
B_train = B_train.drop_duplicates(subset=['ds']) #丢弃重复
C_train = C_train.drop_duplicates(subset=['ds']) #丢弃重复
D_train = D_train.drop_duplicates(subset=['ds']) #丢弃重复


# In[ ]:


print(new_train_data_1)


# In[ ]:


print(A_train)


# In[ ]:


#将异常值赋值为NAN
new_train_data_1['y']  = new_train_data_1['y'].where(new_train_data_1['y'] > 0 , np.nan)
A_train['y']  = A_train['y'].where(A_train['y'] > 0 , np.nan)
B_train['y']  = B_train['y'].where(B_train['y'] > 0 , np.nan)
C_train['y']  = C_train['y'].where(C_train['y'] > 0 , np.nan)
D_train['y']  = D_train['y'].where(D_train['y'] > 0 , np.nan)


# In[ ]:


new_train_data_1 = new_train_data_1.fillna(method = "bfill").fillna(method = "pad")
A_train = A_train.fillna(method = "bfill").fillna(method = "pad")
B_train = B_train.fillna(method = "bfill").fillna(method = "pad")
C_train = C_train.fillna(method = "bfill").fillna(method = "pad")
D_train = D_train.fillna(method = "bfill").fillna(method = "pad")


# In[ ]:


"""function to detect outliers based on the predictions of a model"""
def find_outliers(model, X, y, sigma=3):
    # predict y values using model
    try:
        y_pred = pd.Series(model.predict(X), index=y.index)
    # if predicting fails, try fitting the model first
    except:
        model.fit(X, y)
        y_pred = pd.Series(model.predict(X), index=y.index)
 
    # calculate residuals between the model prediction and true y values
    resid = y - y_pred
    mean_resid = resid.mean()
    std_resid = resid.std()
 
    # calculate z statistic, define outliers to be where |z|>sigma
    z = (resid - mean_resid) / std_resid
    #找出方差大于3的数据的索引，然后丢掉
    outliers = z[abs(z) > sigma].index
 
    # print and plot the results
    print('score=', model.score(X, y))
    
    print("mse=", mean_squared_error(y, y_pred))
    print('---------------------------------------')
 
    print('mean of residuals:', mean_resid)
    print('std of residuals:', std_resid)
    print('---------------------------------------')
 
    print(len(outliers), 'outliers:')
    print(outliers.tolist())
 
    plt.figure(figsize=(15, 5))
    ax_131 = plt.subplot(1,3,1)
    plt.plot(y,y_pred,'.')
    plt.plot(y.loc[outliers],y_pred.loc[outliers],'ro')
    plt.legend(['Accepted','Outlier'])
    plt.xlabel('y')
    plt.ylabel('y_pred');
    
    ax_132=plt.subplot(1,3,2)
    plt.plot(y,y-y_pred,'.')
    plt.plot(y.loc[outliers],y.loc[outliers]-y_pred.loc[outliers],'ro')
    plt.legend(['Accepted','Outlier'])
    plt.xlabel('y')
    plt.ylabel('y - y_pred');
    
    ax_133 = plt.subplot(1,3,3)
    z.plot.hist(bins=50,ax=ax_133)
    z.loc[outliers].plot.hist(color = 'r',bins = 50,ax = ax_133)
    plt.legend(['Accepted','Outlier'])
    plt.xlabel('z')
    plt.savefig('outliers.png')
    
    return outliers
 


# In[ ]:


X_train=new_train_data_1.iloc[:,0:-1]
y_train=new_train_data_1.iloc[:,-2]
outliers = find_outliers(Ridge(),X_train,y_train)


# In[ ]:


X_train=A_train.iloc[:,0:-1]
y_train=A_train.iloc[:,-2]
outliers = find_outliers(Ridge(),X_train,y_train)


# In[ ]:


X_train=B_train.iloc[:,0:-1]
y_train=B_train.iloc[:,-2]
outliers = find_outliers(Ridge(),X_train,y_train)
for col in outliers:
    B_train['y'][col]=np.nan


# In[ ]:


print(B_train)


# In[ ]:


X_train=C_train.iloc[:,0:-1]
y_train=C_train.iloc[:,-2]
outliers = find_outliers(Ridge(),X_train,y_train)
for col in outliers:
    C_train['y'][col]=np.nan


# In[ ]:


X_train=D_train.iloc[:,0:-1]
y_train=D_train.iloc[:,-2]
outliers = find_outliers(Ridge(),X_train,y_train)
for col in outliers:
    D_train['y'][col]=np.nan


# In[ ]:


new_train_data_1 = new_train_data_1.fillna(method = "bfill").fillna(method = "pad")
A_train = A_train.fillna(method = "bfill").fillna(method = "pad")
B_train = B_train.fillna(method = "bfill").fillna(method = "pad")
C_train = C_train.fillna(method = "bfill").fillna(method = "pad")
D_train = D_train.fillna(method = "bfill").fillna(method = "pad")


# In[ ]:


print(y_train)


# In[ ]:


print(new_train_data_1)


# In[ ]:


fig =plt.figure(figsize=(4,6))
sns.boxplot(new_train_data_1['y'],orient='v',width=0.5)


# In[ ]:


#A_train=handleOutlier(A_train)
fig =plt.figure(figsize=(4,6))
sns.boxplot(A_train['y'],orient='v',width=0.5)


# In[ ]:


#B_train=handleOutlier(B_train)
fig =plt.figure(figsize=(4,6))
sns.boxplot(B_train['y'],orient='v',width=0.5)


# In[ ]:


#C_train=handleOutlier(C_train)
fig =plt.figure(figsize=(4,6))
sns.boxplot(C_train['y'],orient='v',width=0.5)


# In[ ]:


#D_train=handleOutlier(D_train)
fig =plt.figure(figsize=(4,6))
sns.boxplot(D_train['y'],orient='v',width=0.5)


# In[ ]:


print(new_train_data_1['y'].max())
print(new_train_data_1['y'].min())


# In[ ]:


print(D_train)


# In[ ]:


new_train_data_1.head


# In[ ]:


new_train_data_1['add_A'] = np.nan
new_train_data_1['add_B'] = np.nan
new_train_data_1['add_C'] = np.nan
new_train_data_1['add_D'] = np.nan


# In[ ]:


train_y_new=pd.DataFrame()
train_y_new['ds'] = A_train['ds']
train_y_new['y'] = A_train['y']    
mapping = train_y_new.set_index('ds',).squeeze()
new_train_data_1['add_A'].fillna(new_train_data_1['ds'].map(mapping),inplace=True)


# In[ ]:


train_y_new=pd.DataFrame()
train_y_new['ds'] = C_train['ds']
train_y_new['y'] = C_train['y']    
mapping = train_y_new.set_index('ds',).squeeze()
new_train_data_1['add_C'].fillna(new_train_data_1['ds'].map(mapping),inplace=True)


# In[ ]:


train_y_new=pd.DataFrame()
train_y_new['ds'] = D_train['ds']
train_y_new['y'] = D_train['y']    
mapping = train_y_new.set_index('ds',).squeeze()
new_train_data_1['add_D'].fillna(new_train_data_1['ds'].map(mapping),inplace=True)


# In[ ]:


train_y_new=pd.DataFrame()
train_y_new['ds'] = B_train['ds']
train_y_new['y'] = B_train['y']    
mapping = train_y_new.set_index('ds',).squeeze()
new_train_data_1['add_B'].fillna(new_train_data_1['ds'].map(mapping),inplace=True)


# In[ ]:


print(new_train_data_1)


# In[ ]:


print(A_train)


# In[ ]:


#new_train_data['y'].loc[new_train_data['y'] < 1 ] = new_train_data['y'].mean()
print(new_train_data_1['y'].max())
print(new_train_data_1['y'].min())
#new_train_data[new_train_data['y'] < -50] = new_train_data.mean()


# In[ ]:


'''
train_data_2['y']=new_train_data['y']
feature_columns = [col for col in train_data_2.columns if col not in ['ds']]
min_max_scaler = preprocessing.MinMaxScaler()
min_max_scaler = min_max_scaler.fit(train_data_2[feature_columns])
train_data_scaler = min_max_scaler.transform(train_data_2[feature_columns])

train_data_scaler=pd.DataFrame(train_data_scaler)
train_data_scaler.rename(columns={0:'y'}, inplace = True)
train_data_scaler['ds']=new_train_data['ds']
train_data_scaler.head
'''


# In[ ]:


m =     Prophet(growth='linear',                            #趋势默认为线性增长'linear', 还有 'logistic'
       changepoints=None,                              # 默认不指定改变点，prophet会根据趋势自动搜寻潜在的改变点
       n_changepoints=25,                              #如果changepoints设置了，该参数不起作用；如果changepoints没有设置，那么改变点将在changepointrange范围内搜寻。默认设置为25
       changepoint_range=0.8,                    #changepoint_range: 范围0-1，表示改变点落在趋势中的可能的范围
       yearly_seasonality=True,                # 即数据存在年度周期性变化，拟合年度趋势。参数有'auto', 参数有'auto'、True和Fals
       weekly_seasonality=False,                    #同上
       #daily_seasonality=True,
       holidays=None,                                 #默认为None，如果要指定传入节假日数据，数据格式为dataframe，有两个指定的字段holiday和ds，其中holiday列的数据为字符串， ds列为date数据类型
       seasonality_mode='multiplicative',                   #趋势模式有'additive'和'multiplicative'，默认为加法'additive'
       seasonality_prior_scale=10,                  #趋势缩放系数，如果该参数大，会让后续趋势波动变大。如果系数较小，会抑制趋势甚至熨平趋势。默认参数设置为10
       #holidays_prior_scale=10.0,                     #类似于seasonalitypriorscale参数的作用，主要用于解释节假日对趋势波动幅度起到熨平或者加大波动强度对作用。默认也是10
       changepoint_prior_scale=0.5,                  # 类似于seasonalitypriorscale参数的作用，主要用于解释改变点对趋势波动幅度起到熨平或者加大波动强度对作用。默认是0.05
       mcmc_samples=0,                                #参数为整数。如果大于0， 模型使用贝叶斯推断；如果设置为0，使用最大后验概率估计（MAP）
       interval_width=0.8,                         #浮点数类型，为预测提供不确定性区间的宽度。如果mcmcsamples = 0，则模型使只使用MAP估计得出的趋势中的不确定性；如果mcmc.samples> 0，则将对所有模型参数进行积分，其中将包括季节性不确定性。
       uncertainty_samples=2000)                 #用于估计不确定性间隔的模拟次数，默认为1000
m.add_regressor('add_A')
m.add_regressor('add_B')
m.add_regressor('add_C')
m.add_regressor('add_D')

#new_train_data_1['cap'] = 92
#new_train_data_1['floor'] = 32 
m.add_seasonality(name='monthly', period=30.5, fourier_order=10)
forecast = m.fit(new_train_data_1).predict()
print("mse is",mean_squared_error(new_train_data_1['y'].values,forecast['yhat'].values[:len(new_train_data_1)]))


# In[ ]:


print(forecast)


# In[ ]:


def outlier_detection(forecast):
    index = np.where((forecast["yhat"] <= forecast["yhat_lower"])|
                     (forecast["yhat"] >= forecast["yhat_upper"]),True,False)
    return index
outlier_index = outlier_detection(forecast)
outlier_df = new_train_data_1[outlier_index]
print("异常值的数量为:",np.sum(outlier_index))


# In[ ]:


new_train_data_2 = new_train_data_1.copy()


# In[ ]:


#new_train_data_2 = new_train_data_2.drop(outlier_df.index) 删除index


# In[ ]:


from prophet.plot import add_changepoints_to_plot
fig = m.plot(forecast)
a = add_changepoints_to_plot(fig.gca(), m, forecast)


# In[ ]:


future  = m.make_future_dataframe(periods=1461,freq='d',include_history = True)
#from prophet.diagnostics import cross_validation
#df_cv = cross_validation(m, initial='2920 days', period='180 days', horizon = '365 days' )


# In[ ]:


#df_cv.head()
print(future)


# In[ ]:


#from prophet.diagnostics import performance_metrics
#df_p = performance_metrics(df_cv)
#df_p.head()


# In[ ]:


#from prophet.plot import plot_cross_validation_metric
#fig = plot_cross_validation_metric(df_cv, metric='mae')


# In[ ]:


print(future)


# In[ ]:


future['add_A'] = np.nan
future['add_B'] = np.nan
future['add_C'] = np.nan
future['add_D'] = np.nan


# In[ ]:


print(future)


# In[ ]:


future = A_train.copy()


# In[ ]:


print(future)


# In[ ]:


future.rename(columns={'y':'add_A'}, inplace=True)#将原来的命名修改为算法要求的命名
#future['add_A'] = A_train['y']
future['add_B'] = B_train['y']
future['add_C'] = C_train['y']
future['add_D'] = D_train['y']


# In[ ]:


#open_day='1995-01-01'
#close_day='2020-01-01'
#con1=future['ds']>=open_day
#con2=future['ds']<close_day
#future=future[con1&con2]


# In[ ]:


#future['add_A'] = A_train['y']


# In[ ]:


print(A_train)


# In[ ]:


#future['ds'] =future['ds'].apply(lambda x: x.strftime('%Y-%m-%d'))


# In[ ]:


print(future['ds'].dtype)


# In[ ]:


print(future['ds'].dtype)


# In[ ]:


#train_y_new1=pd.DataFrame()
#train_y_new1['ds'] = A_train['ds']
#train_y_new1['y'] = A_train['y']    
mapping = A_train.set_index('ds',).squeeze()
future['add_A'].fillna(future['ds'].map(mapping),inplace=True)


# In[ ]:


print(mapping)


# In[ ]:


#train_y_new1=pd.DataFrame()
#train_y_new1['ds'] = C_train['ds']
#train_y_new1['y'] = C_train['y']    
mapping = B_train.set_index('ds',).squeeze()
future['add_B'].fillna(future['ds'].map(mapping),inplace=True)


# In[ ]:


#train_y_new1=pd.DataFrame()
#train_y_new1['ds'] = D_train['ds']
#train_y_new1['y'] = D_train['y']    
mapping = C_train.set_index('ds',).squeeze()
future['add_C'].fillna(future['ds'].map(mapping),inplace=True)


# In[ ]:


#train_y_new1=pd.DataFrame()
#train_y_new1['ds'] = B_train['ds']
#train_y_new1['y'] = B_train['y']    
mapping = D_train.set_index('ds',).squeeze()
future['add_D'].fillna(future['ds'].map(mapping),inplace=True)


# In[ ]:


print(mapping)


# In[ ]:


print(future)


# In[ ]:


future = future.fillna(method = "bfill").fillna(method = "pad")


# In[ ]:


future.isnull().sum()


# In[ ]:


#future = m.make_future_dataframe(periods=1462,freq='d')
forecast = m.predict(future)


# In[ ]:


print( future)


# In[ ]:


forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']].tail()


# In[ ]:


fig1 = m.plot(forecast)


# In[ ]:


open_day='2016-01-01'
close_day='2020-01-01'
con1=forecast['ds']>=open_day
con2=forecast['ds']<close_day
forecast2=forecast[con1&con2]
forecast2.rename(columns={'yhat':'value'}, inplace=True)#将原来的命名修改为算法要求的命名
print(forecast2)


# In[ ]:


print(forecast2['value'].max())
print(forecast2['value'].min())


# In[ ]:


'''
forecast3=pd.DataFrame()
forecast3['value']=forecast2['value']
forecast3=min_max_scaler.inverse_transform(forecast3)

forecast3=pd.DataFrame(forecast3)
forecast3.rename(columns={0:'value'}, inplace = True)
forecast3.head
'''


# In[ ]:


forecast3 =forecast2.round(1)
forecast3['value'].to_csv('./weather_further_155.csv',index=False)


# In[ ]:


forecast_mean['c']=round(forecast_mean['c'],1)
forecast_mean['y']=round(forecast_mean['value'],1)

forecast3=pd.DataFrame()
forecast3['value'] = forecast_mean['c']
forecast3['value_2'] = forecast_mean['y']
forecast3.to_csv('./weather_further.csv',index=False)


# In[ ]:


print(forecast3['value'].max())
print(forecast3['value'].min())


# In[ ]:


print(forecast3['value_2'].max())
print(forecast3['value_2'].min())


# In[ ]:


forecast3['value_2'].to_csv('./weather_further_new.csv',index=False)




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from pycaret.datasets import get_data
# 从本地加载数据，注意dataset是数据的文件名
data = get_data(dataset='./datasets/diabetes', verbose=False)
# 从pycaret开源仓库下载公开数据
# data = get_data('diabetes', verbose=False)


# In[ ]:


# 查看数据类型和数据维度
type(data), data.shape


# In[ ]:


(pandas.core.frame.DataFrame, (768, 9))


# In[ ]:


# 最后一列表示是否为糖尿病患者，其他列为特征列
data.head()


# In[ ]:


#利用PyCaret核心函数setup，初始化建模环境并准备数据以供模型训练和评估使用：
from pycaret.classification import ClassificationExperiment
s = ClassificationExperiment()
# target目标列，session_id设置随机数种子, preprocesss是否清洗数据，train_size训练集比例, normalize是否归一化数据, normalize_method归一化方式
s.setup(data, target = 'Class variable', session_id = 0, verbose= False, train_size = 0.7, normalize = True, normalize_method = 'minmax')


# In[ ]:


s.get_config()#查看基于setup函数创建的变量：


# In[ ]:


#查看归一化的数据：
s.get_config('X_train_transformed')


# In[ ]:


#绘制某列数据的柱状图：

s.get_config('X_train_transformed')['Number of times pregnant'].hist()


# In[ ]:


#当然也可以利用如下代码创建任务示例来初始化环境：
from pycaret.classification import setup
# s = setup(data, target = 'Class variable', session_id = 0, preprocess = True, train_size = 0.7, verbose = False)


# In[ ]:


#模型训练与评估PyCaret提供了compare_models函数，通过使用默认的10折交叉验证来训练和评估模型库中所有可用估计器的性能：
best = s.compare_models()
# 选择某些模型进行比较
# best = s.compare_models(include = ['dt', 'rf', 'et', 'gbc', 'lightgbm'])
# 按照召回率返回n_select性能最佳的模型
# best_recall_models_top3 = s.compare_models(sort = 'Recall', n_select = 3)


# In[ ]:


#返回当前设置中所有经过训练的模型中的最佳模型:

best_ml = s.automl()
# best_ml


# In[ ]:


# 打印效果最佳的模型
print(best)


# In[ ]:


# 提取所有模型预测结果
models_results = s.pull()
models_results


# In[ ]:


s.plot_model(best, plot = 'confusion_matrix')


# In[ ]:


#如果在jupyter环境，可以通过evaluate_model函数来交互式展示模型的性能：
s.evaluate_model(best)


# In[ ]:


#模型预测
# 预测整个数据集
res = s.predict_model(best, data=data)
# 查看各行预测结果
# res


# In[ ]:


# 预测用于数据训练的测试集
res = s.predict_model(best)


# In[ ]:


# 保存模型到本地
_ = s.save_model(best, 'best_model', verbose = False)
# 导入模型
model = s.load_model( 'best_model')
# 查看模型结构
# model


# In[ ]:


# 预测整个数据集
res = s.predict_model(model, data=data)




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


# 使用逻辑回归进行分类的Baseline代码，附带参数说明
from sklearn.datasets import load_iris
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import accuracy_score, classification_report

# 加载数据集（以鸢尾花数据集为例）
data = load_iris()
X, y = data.data, data.target

# 将数据集拆分为训练集和测试集
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# 初始化逻辑回归模型，并设置常用参数
model = LogisticRegression(
    max_iter=200,  # 最大迭代次数，控制优化器的收敛（常见设置在 100-500 之间）
    solver='lbfgs',  # 优化算法，适用于中小型数据集，其他可选项有：'liblinear', 'saga'
    C=1.0,  # 正则化强度的倒数；较小的值表示较强的正则化（常见设置范围为 0.01-10）
    penalty='l2'  # 正则化类型：'l2' 是默认值，适用于大多数情况
)

# 使用训练数据拟合模型
model.fit(X_train, y_train)

# 使用测试数据进行预测
y_pred = model.predict(X_test)

# 评估模型性能
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy:.2f}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

# 使用随机森林分类器进行分类的Baseline代码，附带参数说明
from sklearn.ensemble import RandomForestClassifier

# 初始化随机森林分类器，并设置常用参数
model_rf = RandomForestClassifier(
    n_estimators=100,  # 森林中树的数量（常见设置在 100-500 之间）
    max_depth=None,  # 每棵树的最大深度，'None' 表示直到所有叶子节点纯净或包含的样本少于 min_samples_split
    min_samples_split=2,  # 内部节点再分裂所需的最小样本数（常见设置在 2-10 之间）
    random_state=42  # 随机种子，用于保证结果的可复现性
)

# 使用训练数据拟合随机森林模型
model_rf.fit(X_train, y_train)

# 使用测试数据进行预测
y_pred_rf = model_rf.predict(X_test)

# 评估随机森林模型性能
accuracy_rf = accuracy_score(y_test, y_pred_rf)
print(f"RandomForest Accuracy: {accuracy_rf:.2f}")
print("RandomForest Classification Report:")
print(classification_report(y_test, y_pred_rf))

# 使用线性回归进行回归的Baseline代码，附带参数说明
from sklearn.datasets import load_boston
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error, r2_score

# 加载数据集（以波士顿房价数据集为例）
data = load_boston()
X, y = data.data, data.target

# 将数据集拆分为训练集和测试集
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# 初始化线性回归模型，并设置常用参数
model_lr = LinearRegression(
    fit_intercept=True,  # 是否计算截距。如果为 False，则不使用截距。
    normalize=False  # 是否在拟合前标准化回归变量。此参数已废弃，默认值为 False。
)

# 使用训练数据拟合线性回归模型
model_lr.fit(X_train, y_train)

# 使用测试数据进行预测
y_pred_lr = model_lr.predict(X_test)

# 评估线性回归模型性能
mse = mean_squared_error(y_test, y_pred_lr)
r2 = r2_score(y_test, y_pred_lr)
print(f"Mean Squared Error: {mse:.2f}")
print(f"R2 Score: {r2:.2f}")

# 使用随机森林回归器进行回归的Baseline代码，附带参数说明
from sklearn.ensemble import RandomForestRegressor

# 初始化随机森林回归器，并设置常用参数
model_rfr = RandomForestRegressor(
    n_estimators=100,  # 森林中树的数量
    max_depth=None,  # 每棵树的最大深度
    min_samples_split=2,  # 内部节点再分裂所需的最小样本数
    random_state=42  # 随机种子，用于保证结果的可复现性
)

# 使用训练数据拟合随机森林回归器模型
model_rfr.fit(X_train, y_train)

# 使用测试数据进行预测
y_pred_rfr = model_rfr.predict(X_test)

# 评估随机森林回归器模型性能
mse_rf = mean_squared_error(y_test, y_pred_rfr)
r2_rf = r2_score(y_test, y_pred_rfr)
print(f"RandomForestRegressor Mean Squared Error: {mse_rf:.2f}")
print(f"RandomForestRegressor R2 Score: {r2_rf:.2f}")

# 使用KMeans聚类的Baseline代码，附带参数说明
from sklearn.cluster import KMeans
from sklearn.datasets import make_blobs

# 生成用于聚类的数据集（以make_blobs为例）
X, _ = make_blobs(n_samples=300, centers=4, cluster_std=0.60, random_state=42)

# 初始化KMeans模型，并设置常用参数
model_kmeans = KMeans(
    n_clusters=4,  # 聚类的簇数，代表最终要分成的类的数量
    init='k-means++',  # 初始化簇中心的方法，'k-means++' 是一种加速收敛的初始化方法
    max_iter=300,  # 最大迭代次数（常见设置为 300-500）
    n_init=10,  # KMeans算法运行的次数，每次以不同的初始簇心运行，最终选择最优结果
    random_state=42  # 随机种子，用于保证结果的可复现性
)

# 使用聚类数据拟合KMeans模型
model_kmeans.fit(X)

# 获取聚类结果
labels = model_kmeans.labels_

# 打印每个样本的聚类标签
print("Cluster labels for each point:")
print(labels)




import torch
import torch.nn as nn
import torch.nn.functional as F
from torch.nn import TransformerEncoder, TransformerEncoderLayer
import math

# 设置设备
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
print("Using device:", device)
print(torch.cuda.is_available())
print(torch.cuda.device_count())
print(torch.cuda.current_device())
print(torch.cuda.get_device_name(0))

class SignalNormalization(nn.Module):
    def __init__(self):
        super(SignalNormalization, self).__init__()

    def forward(self, x):
        max_val, _ = torch.max(torch.abs(x).view(x.size(0), -1), dim=1, keepdim=True)
        max_val = max_val.view(x.size(0), 1, 1, 1, 1)
        return x / (max_val + 1e-8)

class SignalTransformer(nn.Module):
    def __init__(self, d_model, nhead, num_encoder_layers, streams, dim_feedforward=2048, dropout=0.1):
        super(SignalTransformer, self).__init__()
        encoder_layers = TransformerEncoderLayer(d_model, nhead, dim_feedforward, dropout).to(device)
        self.transformer_encoder = TransformerEncoder(encoder_layers, num_encoder_layers).to(device)
        self.pos_encoder = PositionalEncoding(d_model, dropout).to(device)
        self.input_proj = nn.Linear(4 * streams, d_model).to(device)

    def forward(self, x):
        x = self.input_proj(x)
        x = self.pos_encoder(x)
        output = self.transformer_encoder(x)
        return output

class PositionalEncoding(nn.Module):
    def __init__(self, d_model, dropout=0.1, max_len=50):
        super(PositionalEncoding, self).__init__()
        self.dropout = nn.Dropout(p=dropout)
        pe = torch.zeros(max_len, d_model).to(device)
        position = torch.arange(0, max_len, dtype=torch.float).unsqueeze(1).to(device)
        div_term = torch.exp(torch.arange(0, d_model, 2).float() * (-math.log(10000.0) / d_model)).to(device)
        pe[:, 0::2] = torch.sin(position * div_term)
        pe[:, 1::2] = torch.cos(position * div_term)
        pe = pe.unsqueeze(0).transpose(0, 1)
        self.register_buffer('pe', pe)

    def forward(self, x):
        x = x + self.pe[:x.size(0), :]
        return self.dropout(x)

class Neural_receiver(nn.Module):
    def __init__(self, subcarriers, timesymbols, streams, num_bits_per_symbol, num_blocks, base_channels, scene, d_model, nhead, num_encoder_layers, dropout_rate=0.1):
        super(Neural_receiver, self).__init__()
        self.subcarriers = subcarriers
        self.timesymbols = timesymbols
        self.streams = streams
        self.num_blocks = num_blocks
        self.base_channels = base_channels
        self.num_bits_per_symbol = num_bits_per_symbol       
        self.normalization = SignalNormalization().to(device)
        self.signal_transformer = SignalTransformer(d_model, nhead, num_encoder_layers, streams, dropout=dropout_rate).to(device)
        final_output_size = streams * num_bits_per_symbol * timesymbols * subcarriers
        self.output_proj = nn.Linear(d_model * timesymbols * subcarriers, final_output_size).to(device)

        self.register_buffer('W', torch.full((1, 1, timesymbols, subcarriers, 2), 0.9 if scene == 'scene1' else 0.6).to(device))
        self.register_buffer('V', torch.full((1, 1, timesymbols, subcarriers, 2), 0.1 if scene == 'scene1' else 0.0).to(device))

    def forward(self, y, template_pilot):
        y = self.normalization(y.to(device))
        template_pilot = self.normalization(template_pilot.to(device))
        y_weighted = y * self.W
        pilot_weighted = template_pilot * self.V
        batch_size = y.shape[0]
        y = y_weighted.permute(0, 2, 3, 1, 4).reshape(batch_size, self.timesymbols * self.subcarriers, self.streams * 2)
        template_pilot = pilot_weighted.permute(0, 2, 3, 1, 4).reshape(batch_size, self.timesymbols * self.subcarriers, self.streams * 2)
        z = torch.cat([y, template_pilot], dim=-1)
        z = self.signal_transformer(z)
        z = z.reshape(batch_size, -1)
        z = self.output_proj(z)
        z = z.reshape(batch_size, self.streams, self.timesymbols, self.subcarriers, self.num_bits_per_symbol)

        z = z.permute(0, 1, 2, 3, 4)  # 调整维度以匹配目标形状

        return z


#!/usr/bin/env python
# coding: utf-8

# # 赛事背景
# 
# 房屋租赁市场是房地产市场的重要组成部分。中国城市化进程的加剧，导致一二线城市房价不断攀升，越来越多的人选择以房屋租赁的方式来满足住房的需求。价格是反映一定时期内房屋租赁价格水平变动趋势和变动程度，分析预测房屋租赁价格，对于发展完善房屋租赁市场有着重要的意义。
# 
# 据统计，中国有近2亿租房人口，租户偏好千变万化，房源种类各不相同。如何高效且合理的解决房屋价值预估，成为各大平台关注的问题。某租房平台将部分房屋信息数据开放，诚邀大家帮助他们建立价格预测模型来预测房屋租赁价格（敏感信息已脱敏）。
# 
# ## 赛事任务
# 
# 给定某租房平台实际业务中的相关租房信息，包含31个与房屋相关的字段，其中“房屋租金”字段为房屋信息的真实基本租金，即不包含服务费、电费、水费和燃气费等。任务目标是通过训练集训练模型，来预测测试集中“房屋租金”字段的具体值，以此为依据，提高房屋价值预估准确率。
# 
# ## 数据说明
# 
# 赛题数据由训练集和测试集组成，总数据量超过30w，包含31个特征字段。为了保证比赛的公平性，将会从中抽取20万条作为训练集，5万条作为测试集，同时会对部分字段信息进行脱敏。
# 
# ```
# ID、区域1、区域2、区域3、街道、上传日期、服务费、供暖费用、电力基础价格、有阳台、没有停车位、有厨房、有地窖、居住面积、房屋状况、内饰质量、可带宠物、加热类型、有电梯、房屋类型、邮政编码、房间数量、所处楼层、建筑楼层、有花园、最后翻新年份、是新建筑、建成年份、价格趋势、上传图片、房屋租金
# ```

# # 数据读取

# In[ ]:


import pandas as pd
import seaborn as sns

get_ipython().run_line_magic('pylab', 'inline')


# In[ ]:


train_df = pd.read_csv('train.csv')
test_df = pd.read_csv('test.csv')


# In[ ]:


train_df.head()


# # 数据分析

# ## 房屋租金

# In[ ]:


train_df['房屋租金'].max()


# In[ ]:


train_df[train_df['房屋租金'] > train_df['房屋租金'].quantile(0.999)].head(1)


# ## 缺失值分析

# In[ ]:


train_df.isnull().mean(0)


# ## 相关性分析

# In[ ]:


train_df.corr()['房屋租金']


# ## 探索性分析

# In[ ]:


train_df.groupby(['区域1'])['房屋租金'].mean().plot(kind='bar')


# In[ ]:


train_df.groupby(['有花园'])['房屋租金'].mean()


# # 特征编码

# In[ ]:


train_df.dtypes


# In[ ]:


train_df['上传日期_day'] = train_df['上传日期'].apply(lambda x: int(x[-2:]))
test_df['上传日期_day'] = test_df['上传日期'].apply(lambda x: int(x[-2:]))


# In[ ]:


train_df['上传日期_month'] = train_df['上传日期'].apply(lambda x: x[:2])
test_df['上传日期_month'] = test_df['上传日期'].apply(lambda x: x[:2])

train_df['上传日期_month'] = train_df['上传日期_month'].apply(lambda x: ['Fe', 'Ma', 'Oc', 'Se'].index(x))
test_df['上传日期_month'] = test_df['上传日期_month'].apply(lambda x: ['Fe', 'Ma', 'Oc', 'Se'].index(x))

train_df.drop(['上传日期'], axis=1, inplace=True)
test_df.drop(['上传日期'], axis=1, inplace=True)


# In[ ]:


train_df['可带宠物'] = train_df['可带宠物'].map({'negotiable': 0.5, 'no': 0, 'yea': 1})
test_df['可带宠物'] = test_df['可带宠物'].map({'negotiable': 0.5, 'no': 0, 'yea': 1})


# In[ ]:


for col in train_df.select_dtypes(bool).columns:
    train_df[col] = train_df[col].astype(int)
    test_df[col] = test_df[col].astype(int)


# In[ ]:


train_df.dtypes


# In[ ]:


train_df['房屋租金'] = np.log1p(train_df['房屋租金'])


# In[ ]:


train_df['区域1_租金价格'] = train_df['区域1'].map(train_df.groupby(['区域1'])['房屋租金'].mean())
test_df['区域1_租金价格'] = test_df['区域1'].map(train_df.groupby(['区域1'])['房屋租金'].mean())

train_df['区域2_租金价格'] = train_df['区域2'].map(train_df.groupby(['区域2'])['房屋租金'].mean())
test_df['区域2_租金价格'] = test_df['区域2'].map(train_df.groupby(['区域2'])['房屋租金'].mean())

train_df['邮政编码_租金价格'] = train_df['邮政编码'].map(train_df.groupby(['邮政编码'])['房屋租金'].mean())
test_df['邮政编码_租金价格'] = test_df['邮政编码'].map(train_df.groupby(['邮政编码'])['房屋租金'].mean())


# In[ ]:


train_df['所处楼层_div'] = train_df['所处楼层'] / train_df['建筑楼层']
test_df['所处楼层_div'] = test_df['所处楼层'] / test_df['建筑楼层']

train_df['所处楼层_max'] = (train_df['所处楼层'] == train_df['建筑楼层']).astype(int)
test_df['所处楼层_max'] = (test_df['所处楼层'] == test_df['建筑楼层']).astype(int)

train_df['居住面积_mean'] = train_df['居住面积'] / train_df['房间数量']
test_df['居住面积_mean'] = test_df['居住面积'] / test_df['房间数量']


# In[ ]:


train_df['建成年龄'] = 2022 - train_df['建成年份']
test_df['建成年龄'] = 2022 - test_df['建成年份']

train_df['翻新年龄'] = 2022 - train_df['最后翻新年份']
test_df['翻新年龄'] = 2022 - test_df['最后翻新年份']


# # 搭建模型

# In[ ]:


for col in ['区域1', '区域2', '区域3', '街道', '房屋类型', '房屋状况', '可带宠物', '邮政编码']:
    train_df[col] = train_df[col].astype('category')
    test_df[col] = test_df[col].astype('category')


# In[ ]:


# 模型交叉验证
def run_model_cv(model, kf, X_tr, y, X_te, cate_col=None):
    train_pred = np.zeros( len(X_tr) )
    test_pred = np.zeros( len(X_te)  )

    cv_clf = []
    for tr_idx, val_idx in kf.split(X_tr, y):
        x_tr = X_tr.iloc[tr_idx]; y_tr = y.iloc[tr_idx]

        x_val = X_tr.iloc[val_idx]; y_val = y.iloc[val_idx]

        call_back = [
            lgb.early_stopping(50),
        ]
        eval_set = [(x_val, y_val)]
        model.fit(x_tr, y_tr, eval_set=eval_set, callbacks=call_back, verbose=-1,
                 categorical_feature= 'auto', eval_metric= 'mae', )

        cv_clf.append(model)

        train_pred[val_idx] = model.predict(x_val)
        test_pred += model.predict(X_te)

    test_pred /= kf.n_splits
    
    print(np.abs(np.exp(train_pred) - np.exp(y)).mean())
    return train_pred, test_pred, cv_clf


# In[ ]:


from sklearn.model_selection import KFold
import lightgbm as lgb

test_pred_tta = np.zeros( len(test_df)  )
for _ in range(1):
    clf = lgb.LGBMRegressor(
        max_depth=7, 
        n_estimators=4000, 
        n_jobs=-1, 
        verbose=-1,
        verbosity=-1,
        learning_rate=0.03,
        random_state=np.random.randint(100)
    )

    train_pred, test_pred, cv_clf = run_model_cv(
        clf, KFold(n_splits=5, random_state=np.random.randint(100), shuffle=True),
        train_df.drop(['ID', '房屋租金'], axis=1),
        train_df['房屋租金'],
        test_df.drop(['ID'], axis=1),
    )
    
    test_pred_tta += test_pred

# 133.28688613802427，95.7
# 132.1 75.0


# In[ ]:


pd.DataFrame({
    'ID': test_df['ID'],
    '房屋租金': np.exp(test_pred_tta / 1).astype(int)
}).to_csv('submit.csv', index=None)


# In[ ]:







#1. ARIMA 模型的时间序列预测 Baseline
import pandas as pd
from statsmodels.tsa.arima.model import ARIMA
import matplotlib.pyplot as plt

# 加载时间序列数据集（假设数据是月份和销售额的时间序列）
data = pd.read_csv('time_series_data.csv', index_col='Month', parse_dates=True)
time_series = data['Sales']

# 拆分训练和测试数据
train_data = time_series[:'2020-12']
test_data = time_series['2021-01':]

# 建立并拟合ARIMA模型
model = ARIMA(train_data, order=(5, 1, 0))  # (p, d, q) 参数：5阶自回归，1阶差分，0阶移动平均
model_fit = model.fit()

# 进行预测
forecast = model_fit.forecast(steps=len(test_data))

# 绘图
plt.figure(figsize=(10, 5))
plt.plot(train_data, label='Train')
plt.plot(test_data, label='Test')
plt.plot(test_data.index, forecast, label='Forecast', linestyle='--')
plt.legend()
plt.show()
'''
参数说明：
order (p, d, q)：ARIMA的三个参数，分别表示自回归阶数、差分次数、移动平均阶数。
model.fit()：拟合模型。
model.forecast()：对未来进行预测。
'''
#2. Prophet 模型的时间序列预测 Baseline
from fbprophet import Prophet
import pandas as pd
import matplotlib.pyplot as plt

# 加载时间序列数据集
data = pd.read_csv('time_series_data.csv')
data.columns = ['ds', 'y']  # Prophet需要特定格式，时间列必须为 'ds'，目标列必须为 'y'

# 初始化并拟合模型
model = Prophet()
model.fit(data)

# 创建未来日期的DataFrame，进行预测
future = model.make_future_dataframe(periods=12, freq='M')  # 预测未来12个月
forecast = model.predict(future)

# 绘制预测结果
model.plot(forecast)
plt.show()
'''
参数说明：
ds：时间列，必须是日期时间格式。
y：预测的目标变量。
periods：预测的时间长度。
freq：时间间隔，这里设为'M'表示按月进行预测。
'''
#3. 基于LSTM的时间序列预测 Baseline
import numpy as np
import pandas as pd
from sklearn.preprocessing import MinMaxScaler
from keras.models import Sequential
from keras.layers import LSTM, Dense

# 加载时间序列数据集
data = pd.read_csv('time_series_data.csv', index_col='Month', parse_dates=True)
time_series = data['Sales']

# 数据标准化
scaler = MinMaxScaler(feature_range=(0, 1))
scaled_data = scaler.fit_transform(time_series.values.reshape(-1, 1))

# 准备训练集和测试集
train_size = int(len(scaled_data) * 0.8)
train_data, test_data = scaled_data[:train_size], scaled_data[train_size:]

# 创建输入序列和输出序列
def create_dataset(data, time_step=1):
    X, y = [], []
    for i in range(len(data) - time_step):
        X.append(data[i:(i + time_step), 0])
        y.append(data[i + time_step, 0])
    return np.array(X), np.array(y)

time_step = 10  # 设置时间步长
X_train, y_train = create_dataset(train_data, time_step)
X_test, y_test = create_dataset(test_data, time_step)

# 调整输入数据的形状为LSTM所需的格式：[样本数, 时间步长, 特征数]
X_train = X_train.reshape(X_train.shape[0], X_train.shape[1], 1)
X_test = X_test.reshape(X_test.shape[0], X_test.shape[1], 1)

# 构建LSTM模型
model = Sequential()
model.add(LSTM(50, return_sequences=True, input_shape=(time_step, 1)))  # 第一层LSTM
model.add(LSTM(50, return_sequences=False))  # 第二层LSTM
model.add(Dense(1))  # 输出层

# 编译模型
model.compile(optimizer='adam', loss='mean_squared_error')

# 训练模型
model.fit(X_train, y_train, epochs=10, batch_size=32, validation_data=(X_test, y_test))

# 进行预测
predicted = model.predict(X_test)
predicted = scaler.inverse_transform(predicted)

# 可视化预测结果
import matplotlib.pyplot as plt
plt.figure(figsize=(10, 5))
plt.plot(time_series.index[-len(y_test):], scaler.inverse_transform(y_test.reshape(-1, 1)), label='True Data')
plt.plot(time_series.index[-len(y_test):], predicted, label='Predicted Data', linestyle='--')
plt.legend()
plt.show()
'''
参数说明：
time_step：LSTM的时间步长，用于决定LSTM记忆多少历史信息。
LSTM层：50个神经元的两层LSTM，用于捕获时间序列中的长期依赖关系。
epochs：训练的轮数，决定模型训练的次数。

'''



import torch
import torch.nn as nn

class ResBlock(nn.Module):
    def __init__(self, in_channels, out_channels, stride=1, kernel_size=3, dropout_rate=0.1):
        super(ResBlock, self).__init__()
        self.conv1 = nn.Conv2d(in_channels, out_channels, kernel_size=kernel_size, stride=stride, padding=kernel_size//2, bias=False)
        self.bn1 = nn.BatchNorm2d(out_channels)
        self.relu = nn.ReLU(inplace=True)
        self.dropout = nn.Dropout(dropout_rate)
        self.conv2 = nn.Conv2d(out_channels, out_channels, kernel_size=kernel_size, stride=1, padding=kernel_size//2, bias=False)
        self.bn2 = nn.BatchNorm2d(out_channels)

        if in_channels != out_channels or stride != 1:
            self.downsample = nn.Sequential(
                nn.Conv2d(in_channels, out_channels, kernel_size=1, stride=stride, bias=False),
                nn.BatchNorm2d(out_channels)
            )
        else:
            self.downsample = None

    def forward(self, x):
        identity = x

        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)
        out = self.dropout(out)
        out = self.conv2(out)
        out = self.bn2(out)

        if self.downsample is not None:
            identity = self.downsample(x)

        out += identity
        out = self.relu(out)
        return out

class Neural_receiver(nn.Module):
    def __init__(self, subcarriers, timesymbols, streams, num_bits_per_symbol, num_blocks, base_channels, scene, dropout_rate=0.1):
        super(Neural_receiver, self).__init__()
        self.subcarriers = subcarriers
        self.timesymbols = timesymbols
        self.streams = streams
        self.num_blocks = num_blocks
        self.base_channels = base_channels
        self.num_bits_per_symbol = num_bits_per_symbol

        # 初始化权重
        if scene == 'scene1':
            self.register_buffer('W', torch.full((1, 1, timesymbols, subcarriers, 2), 0.9))
            self.register_buffer('V', torch.full((1, 1, timesymbols, subcarriers, 2), 0.1))
        elif scene == 'scene2':
            self.register_buffer('W', torch.full((1, 1, timesymbols, subcarriers, 2), 0.6))
            self.register_buffer('V', torch.zeros((1, 1, timesymbols, subcarriers, 2)))
            specific_t_V = {1: [0, 4, 8], 2: [1, 5, 9], 3: [2, 6, 10], 4: [3, 7, 11]}
            for l in range(streams):
                for t in specific_t_V[l + 1]:
                    self.V[0, 0, t, :, :] = 1.6

        # 初始化卷积层
        self.init_conv = nn.Conv2d(4 * self.streams, self.base_channels, kernel_size=3, padding=1)
        in_channels = self.base_channels
        # 初始化残差块
        self.blocks = nn.Sequential()
        for block_id in range(self.num_blocks):
            in_channels = self.base_channels if block_id == 0 else layer_channels
            layer_channels = self.base_channels + (block_id // 10) * 32
            self.blocks.add_module(
                name=f'resblock_{block_id}',
                module=ResBlock(in_channels, layer_channels, dropout_rate=dropout_rate)
            )
            in_channels = layer_channels
        final_channels = in_channels  # 或特定残差块设计的输出通道数
        # 初始化最终的卷积层
        self.final_conv = nn.Conv2d(final_channels, streams * num_bits_per_symbol, kernel_size=3, padding=1)

    def preprocess(self, y):
        # 数据预处理步骤：将实部和虚部转换为复数形式
        y_complex = torch.complex(y[..., 0], y[..., 1])
        return y_complex

    def forward(self, y, template_pilot):
        # 在模型前向传播中调用预处理
        #y_complex = self.preprocess(y)
        #template_pilot_complex = self.preprocess(template_pilot)
        # 分别应用权重到实部和虚部
        y_real_weighted = y[..., 0] * self.W[..., 0] - y[..., 1] * self.W[..., 1]  # 应用权重到实部
        y_imag_weighted = y[..., 1] * self.W[..., 0] + y[..., 0] * self.W[..., 1]  # 应用权重到虚部
        template_pilot_real_weighted = template_pilot[..., 0] * self.V[..., 0] - template_pilot[..., 1] * self.V[..., 1]  # 应用权重到实部
        template_pilot_imag_weighted = template_pilot[..., 1] * self.V[..., 0] + template_pilot[..., 0] * self.V[..., 1]  # 应用权重到虚部

        # 组合加权后的实部和虚部
        y_weighted = torch.stack([y_real_weighted, y_imag_weighted], dim=-1)  # 最后一个维度是实部和虚部
        template_pilot_weighted = torch.stack([template_pilot_real_weighted, template_pilot_imag_weighted], dim=-1)

        # 现在 y_weighted 包含了加权后的实部和虚部，您可以转换它为复数形式
        y_weighted_complex = torch.complex(y_weighted[..., 0], y_weighted[..., 1])
        template_pilot_weighted_complex = torch.complex(template_pilot_weighted[..., 0], template_pilot_weighted[..., 1])

        # 应用权重
 

        # 调整数据形状以适应卷积层
        batch_size = y.shape[0]
        # 对于加权后的复数信号，调整维度以适应卷积层，确保考虑所有五个维度
        y = torch.view_as_real(y_weighted_complex).permute(0, 4, 1, 2, 3)  # 注意维度顺序
        # 调整维度，合并最后两个维度
        y = y.reshape(y.shape[0], -1, y.shape[2], y.shape[3])

        # 对于加权后的复数导频信号，执行相同的维度调整
        template_pilot = torch.view_as_real(template_pilot_weighted_complex).permute(0, 4, 1, 2, 3)  # 注意维度顺序
        # 调整维度，合并最后两个维度
        template_pilot = template_pilot.reshape(template_pilot.shape[0], -1, template_pilot.shape[2], template_pilot.shape[3])

        # 将接收信号和导频信号合并为模型的输入
        z = torch.cat([y, template_pilot], dim=1)  # 在通道维度上合并

        # 通过神经网络层
        z = self.init_conv(z)
        z = self.blocks(z)
        z = self.final_conv(z)

        # 将输出调整回原始维度
        z = z.permute(0, 2, 3, 1).view(batch_size, self.timesymbols, self.subcarriers, self.streams, self.num_bits_per_symbol)
        
        return z


#!/usr/bin/env python
# coding: utf-8

# In[ ]:


#1. 图像分类任务的Baseline概述 
#PyTorch是一个灵活且易用的深度学习框架，即使在CPU上也能实现简单的图像分类任务。下面我们会使用经典的MNIST数据集来进行基线模型的搭建，MNIST是一个包含手写数字（0-9）的小型图像数据集，非常适合用于快速训练和调试。

import torch
import torch.nn as nn
import torch.optim as optim
from torch.utils.data import DataLoader
from torchvision import datasets, transforms
# 定义数据转换和加载
transform = transforms.Compose([
    transforms.ToTensor(),  # 转换为Tensor
    transforms.Normalize((0.5,), (0.5,))  # 标准化
])

# 加载训练集和测试集
train_dataset = datasets.MNIST(root='./data', train=True, download=True, transform=transform)
test_dataset = datasets.MNIST(root='./data', train=False, download=True, transform=transform)

train_loader = DataLoader(dataset=train_dataset, batch_size=64, shuffle=True)
test_loader = DataLoader(dataset=test_dataset, batch_size=64, shuffle=False)
#4. 构建简单的卷积神经网络模型
#我们将使用一个小型的卷积神经网络（CNN）作为Baseline模型。这个网络结构包括两个卷积层和两个全连接层，适合在CPU上训练。
# 定义卷积神经网络
class SimpleCNN(nn.Module):
    def __init__(self):
        super(SimpleCNN, self).__init__()
        self.conv1 = nn.Conv2d(in_channels=1, out_channels=16, kernel_size=3, stride=1, padding=1)  # 第一层卷积
        self.conv2 = nn.Conv2d(in_channels=16, out_channels=32, kernel_size=3, stride=1, padding=1)  # 第二层卷积
        self.fc1 = nn.Linear(32 * 7 * 7, 128)  # 全连接层1
        self.fc2 = nn.Linear(128, 10)  # 全连接层2，输出10类

    def forward(self, x):
        x = torch.relu(self.conv1(x))
        x = torch.max_pool2d(x, 2)  # 池化层1
        x = torch.relu(self.conv2(x))
        x = torch.max_pool2d(x, 2)  # 池化层2
        x = x.view(-1, 32 * 7 * 7)  # 展平特征图
        x = torch.relu(self.fc1(x))
        x = self.fc2(x)
        return x

# 初始化模型
model = SimpleCNN()
# 损失函数和优化器
criterion = nn.CrossEntropyLoss()
optimizer = optim.Adam(model.parameters(), lr=0.001)  # 学习率可以根据需要进行调整
# 训练模型
num_epochs = 5

for epoch in range(num_epochs):
    model.train()  # 设置模型为训练模式
    running_loss = 0.0
    for images, labels in train_loader:
        # 前向传播
        outputs = model(images)
        loss = criterion(outputs, labels)

        # 反向传播和优化
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()

        running_loss += loss.item()
    
    print(f'Epoch [{epoch+1}/{num_epochs}], Loss: {running_loss/len(train_loader):.4f}')
# 在测试集上评估模型
model.eval()  # 设置模型为评估模式
correct = 0
total = 0

with torch.no_grad():  # 在评估时不需要计算梯度
    for images, labels in test_loader:
        outputs = model(images)
        _, predicted = torch.max(outputs.data, 1)
        total += labels.size(0)
        correct += (predicted == labels).sum().item()

accuracy = 100 * correct / total
print(f'Test Accuracy: {accuracy:.2f}%')
# 保存模型
torch.save(model.state_dict(), 'simple_cnn_model.pth')

# 载入模型（如需再使用）
model = SimpleCNN()
model.load_state_dict(torch.load('simple_cnn_model.pth'))

# 假设有新的数据需要预测
new_images, _ = next(iter(test_loader))
model.eval()
predictions = model(new_images)
_, predicted_labels = torch.max(predictions, 1)

# 打印部分预测结果
print("Predicted labels:", predicted_labels[:10].numpy())
#1. 基于卷积神经网络（CNN）的图像分类Baseline
#卷积神经网络（CNN）是一种非常经典的用于图像分类的深度学习模型。以下是一个使用CIFAR-10数据集的CNN Baseline代码。
import torch
import torch.nn as nn
import torch.optim as optim
from torch.utils.data import DataLoader
from torchvision import datasets, transforms

# 定义数据预处理和加载
transform = transforms.Compose([
    transforms.ToTensor(),
    transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5))
])

# 加载CIFAR-10数据集
train_dataset = datasets.CIFAR10(root='./data', train=True, download=True, transform=transform)
test_dataset = datasets.CIFAR10(root='./data', train=False, download=True, transform=transform)

train_loader = DataLoader(dataset=train_dataset, batch_size=64, shuffle=True)
test_loader = DataLoader(dataset=test_dataset, batch_size=64, shuffle=False)

# 定义卷积神经网络模型
class SimpleCNN(nn.Module):
    def __init__(self):
        super(SimpleCNN, self).__init__()
        self.conv1 = nn.Conv2d(3, 32, 3, 1, 1)
        self.conv2 = nn.Conv2d(32, 64, 3, 1, 1)
        self.pool = nn.MaxPool2d(2, 2)
        self.fc1 = nn.Linear(64 * 8 * 8, 512)
        self.fc2 = nn.Linear(512, 10)

    def forward(self, x):
        x = torch.relu(self.conv1(x))
        x = self.pool(torch.relu(self.conv2(x)))
        x = x.view(-1, 64 * 8 * 8)
        x = torch.relu(self.fc1(x))
        x = self.fc2(x)
        return x

# 初始化模型、损失函数和优化器
model = SimpleCNN()
criterion = nn.CrossEntropyLoss()
optimizer = optim.Adam(model.parameters(), lr=0.001)

# 训练模型
num_epochs = 5
for epoch in range(num_epochs):
    model.train()
    running_loss = 0.0
    for images, labels in train_loader:
        optimizer.zero_grad()
        outputs = model(images)
        loss = criterion(outputs, labels)
        loss.backward()
        optimizer.step()
        running_loss += loss.item()

    print(f'Epoch [{epoch+1}/{num_epochs}], Loss: {running_loss/len(train_loader):.4f}')

# 评估模型性能
model.eval()
correct = 0
total = 0
with torch.no_grad():
    for images, labels in test_loader:
        outputs = model(images)
        _, predicted = torch.max(outputs.data, 1)
        total += labels.size(0)
        correct += (predicted == labels).sum().item()

accuracy = 100 * correct / total
print(f'Test Accuracy: {accuracy:.2f}%')


# In[ ]:


# 使用PyTorch进行猫狗分类的Baseline代码，附带参数说明
import torch
import torch.nn as nn
import torch.optim as optim
from torch.utils.data import DataLoader, random_split
from torchvision import datasets, transforms

# 定义超参数
batch_size = 32  # 每个批次的样本数
learning_rate = 0.001  # 学习率
num_epochs = 10  # 训练的轮数

# 定义数据转换（包括数据预处理）
transform = transforms.Compose([
    transforms.Resize((64, 64)),  # 调整图像大小到64x64
    transforms.ToTensor(),  # 转换为张量
    transforms.Normalize((0.5,), (0.5,))  # 归一化，使图像像素值在[-1, 1]之间
])

# 加载猫狗数据集（假设使用的是本地文件夹）
dataset = datasets.ImageFolder(root='path/to/cat_dog_data', transform=transform)
train_size = int(0.8 * len(dataset))
test_size = len(dataset) - train_size
train_dataset, test_dataset = random_split(dataset, [train_size, test_size])

# 创建数据加载器
train_loader = DataLoader(train_dataset, batch_size=batch_size, shuffle=True)
test_loader = DataLoader(test_dataset, batch_size=batch_size, shuffle=False)

# 定义简单的卷积神经网络模型
class SimpleCNN(nn.Module):
    def __init__(self):
        super(SimpleCNN, self).__init__()
        self.conv1 = nn.Conv2d(3, 16, 3, padding=1)  # 第一个卷积层，输入通道3，输出通道16，卷积核大小3x3
        self.conv2 = nn.Conv2d(16, 32, 3, padding=1)  # 第二个卷积层
        self.fc1 = nn.Linear(32 * 16 * 16, 128)  # 全连接层，输入大小32*16*16，输出大小128
        self.fc2 = nn.Linear(128, 2)  # 最后一层，输出大小为2（猫和狗）

    def forward(self, x):
        x = torch.relu(self.conv1(x))
        x = torch.max_pool2d(x, 2, 2)  # 最大池化层
        x = torch.relu(self.conv2(x))
        x = torch.max_pool2d(x, 2, 2)
        x = x.view(-1, 32 * 16 * 16)  # 展平
        x = torch.relu(self.fc1(x))
        x = self.fc2(x)
        return x

# 初始化模型、损失函数和优化器
model = SimpleCNN()
criterion = nn.CrossEntropyLoss()  # 交叉熵损失函数
optimizer = optim.Adam(model.parameters(), lr=learning_rate)  # Adam优化器

# 训练模型
for epoch in range(num_epochs):
    model.train()
    running_loss = 0.0
    for images, labels in train_loader:
        # 前向传播
        outputs = model(images)
        loss = criterion(outputs, labels)

        # 反向传播和优化
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()

        running_loss += loss.item()
    print(f"Epoch [{epoch+1}/{num_epochs}], Loss: {running_loss/len(train_loader):.4f}")

# 评估模型性能
model.eval()
correct = 0
total = 0
with torch.no_grad():
    for images, labels in test_loader:
        outputs = model(images)
        _, predicted = torch.max(outputs, 1)
        total += labels.size(0)
        correct += (predicted == labels).sum().item()

accuracy = 100 * correct / total
print(f"Test Accuracy: {accuracy:.2f}%")


# In[ ]:


# 使用PyTorch进行基本图像多分类的Baseline代码，附带参数说明
# 加载CIFAR-10数据集
cifar_transform = transforms.Compose([
    transforms.Resize((32, 32)),  # 调整图像大小到32x32
    transforms.ToTensor(),
    transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5))  # 归一化
])

train_dataset_cifar = datasets.CIFAR10(root='./data', train=True, download=True, transform=cifar_transform)
test_dataset_cifar = datasets.CIFAR10(root='./data', train=False, download=True, transform=cifar_transform)

train_loader_cifar = DataLoader(train_dataset_cifar, batch_size=batch_size, shuffle=True)
test_loader_cifar = DataLoader(test_dataset_cifar, batch_size=batch_size, shuffle=False)

# 定义卷积神经网络用于多分类任务（CIFAR-10）
class CIFARCNN(nn.Module):
    def __init__(self):
        super(CIFARCNN, self).__init__()
        self.conv1 = nn.Conv2d(3, 32, 3, padding=1)
        self.conv2 = nn.Conv2d(32, 64, 3, padding=1)
        self.fc1 = nn.Linear(64 * 8 * 8, 512)
        self.fc2 = nn.Linear(512, 10)  # CIFAR-10有10个分类

    def forward(self, x):
        x = torch.relu(self.conv1(x))
        x = torch.max_pool2d(x, 2, 2)
        x = torch.relu(self.conv2(x))
        x = torch.max_pool2d(x, 2, 2)
        x = x.view(-1, 64 * 8 * 8)
        x = torch.relu(self.fc1(x))
        x = self.fc2(x)
        return x

# 初始化模型、损失函数和优化器
model_cifar = CIFARCNN()
criterion_cifar = nn.CrossEntropyLoss()
optimizer_cifar = optim.Adam(model_cifar.parameters(), lr=learning_rate)

# 训练CIFAR-10模型
for epoch in range(num_epochs):
    model_cifar.train()
    running_loss = 0.0
    for images, labels in train_loader_cifar:
        # 前向传播
        outputs = model_cifar(images)
        loss = criterion_cifar(outputs, labels)

        # 反向传播和优化
        optimizer_cifar.zero_grad()
        loss.backward()
        optimizer_cifar.step()

        running_loss += loss.item()
    print(f"Epoch [{epoch+1}/{num_epochs}], Loss: {running_loss/len(train_loader_cifar):.4f}")

# 评估CIFAR-10模型性能
model_cifar.eval()
correct = 0
total = 0
with torch.no_grad():
    for images, labels in test_loader_cifar:
        outputs = model_cifar(images)
        _, predicted = torch.max(outputs, 1)
        total += labels.size(0)
        correct += (predicted == labels).sum().item()

accuracy_cifar = 100 * correct / total
print(f"CIFAR-10 Test Accuracy: {accuracy_cifar:.2f}%")

# 使用PyTorch进行手写数字识别（MNIST）的Baseline代码，附带参数说明
# 加载MNIST数据集
mnist_transform = transforms.Compose([
    transforms.ToTensor(),
    transforms.Normalize((0.5,), (0.5,))
])

train_dataset_mnist = datasets.MNIST(root='./data', train=True, download=True, transform=mnist_transform)
test_dataset_mnist = datasets.MNIST(root='./data', train=False, download=True, transform=mnist_transform)

train_loader_mnist = DataLoader(train_dataset_mnist, batch_size=batch_size, shuffle=True)
test_loader_mnist = DataLoader(test_dataset_mnist, batch_size=batch_size, shuffle=False)

# 定义简单的全连接神经网络用于MNIST分类
class MNISTNN(nn.Module):
    def __init__(self):
        super(MNISTNN, self).__init__()
        self.fc1 = nn.Linear(28 * 28, 256)
        self.fc2 = nn.Linear(256, 128)
        self.fc3 = nn.Linear(128, 10)

    def forward(self, x):
        x = x.view(-1, 28 * 28)
        x = torch.relu(self.fc1(x))
        x = torch.relu(self.fc2(x))
        x = self.fc3(x)
        return x

# 初始化模型、损失函数和优化器
model_mnist = MNISTNN()
criterion_mnist = nn.CrossEntropyLoss()
optimizer_mnist = optim.Adam(model_mnist.parameters(), lr=learning_rate)

# 训练MNIST模型
for epoch in range(num_epochs):
    model_mnist.train()
    running_loss = 0.0
    for images, labels in train_loader_mnist:

